# look at the tcp connections in a tcpdump output file.
# output looks like:
#
# <hosta> <porta> <seqa> <pktsa> <bytesa>
# <hostb> <portb> <seqb> <pktsb> <bytesb>
# <starttime> <stoptime>
#
# <hosta>, <porta> identifier for this end of the connection
# <seqa> low sequence number for this end of the connection
# <pktsa> packets sent by this end of the connection
# <bytesa> bytes sent by this end of the connection
#
# <hostb>, <portb> the other end of the connection
# <seqb>, <pktsb>, <bytesb> etc.
#
# <starttime> timestamp this connection first seen
# <stoptime> timestamp this connection last seen

$5 !~ /[SFR]/	{
	# no SYNs, no FINs, no RSTs (why not?)

	# source host, port
	n = split($2, array, ".")
	x = ""
	dot = ""
	for (i = 1; i < n; i++) {
		x = sprintf("%s%s%s", x, dot, array[i])
		dot = "."
	}
	shost = x
	sport = array[n]

	# destination host, port
	n = split($4, array, ".")
	x = ""
	dot = ""
	for (i = 1; i < n; i++) {
		x = sprintf("%s%s%s", x, dot, array[i])
		dot = "."
	}
	dhost = x
	n = split(array[n], array, ":")
	dport = array[1]

	# number of data bytes
	if ($6 !~ /^ack/) {
		n = split($6, array, "(")
		n = split(array[2], array, ")")
		bytes = array[1]
	} else {
		bytes = 0
	}

	# sequence numbers
	if ($6 !~ /^ack/) {
		n = split($6, array, ":")
		sequence = array[1]
		ack = $8
	} else {
		sequence = 0
		ack = $7
	}

	# find connection
	if ((conn = connections[dhost dport shost sport])) {
		abytes = 0
		abytes = 0
		bbytes = bytes
		bpkts = 1
		bseq = sequence
		aseq = ack
	} else {
		if ((conn = connections[shost sport dhost dport]) == 0) {
			# create a new connection
			conncnt++
			connections[shost sport dhost dport] = conncnt
			conn = conncnt
			hosta[conn] = shost
			porta[conn] = sport
			hostb[conn] = dhost
			portb[conn] = dport
			starttime[conn] = $1
		}
		abytes = bytes
		apkts = 1
		bbytes = 0
		bpkts = 0
		aseq = sequence
		bseq = ack
	}
	stoptime[conn] = $1
	pktsa[conn] += apkts
	bytesa[conn] += abytes
	pktsb[conn] += bpkts
	bytesb[conn] += bbytes
	if (seqa[conn] == 0) {
		seqa[conn] = aseq
	}
	if (seqb[conn] == 0) {
		seqb[conn] = bseq
	}
#	printf("%s %s %d %d %d ",
#		hosta[conn], porta[conn], seqa[conn],
#		pktsa[conn], bytesa[conn])
#	printf("%s %s %d %d %d ",
#		hostb[conn], portb[conn], seqb[conn],
#		pktsb[conn], bytesb[conn])
#	printf("%s %s\n", starttime[conn], stoptime[conn])
}

END {
# do output
# <hosta> <porta> <seqa> <pktsa> <bytesa>
# <hostb> <portb> <seqb> <pktsb> <bytesb>
# <starttime> <stoptime>

	for (conn = 1; conn <= conncnt; conn++) {
		printf("%s %s %d %d %d ",
			hosta[conn], porta[conn], seqa[conn],
			pktsa[conn], bytesa[conn])
		printf("%s %s %d %d %d ",
			hostb[conn], portb[conn], seqb[conn],
			pktsb[conn], bytesb[conn])
		printf("%s %s\n", starttime[conn], stoptime[conn])
	}
}
