# scan a tcpdump output (consisting solely of TCP connections) looking
# for all the hosts listed in the file.  The output looks like:
#
# <host> <pktssent> <pktsrcvd> <bytessent> <bytesrcvd> <starttime> <stoptime>
#
# <host> is the host.
# <pktssent> is the number of packets sent *from* that host.
# <pktsrcvd> is the number of packets sent *to* that host.
# <bytessent> is the number of data bytes sent from that host.
# <bytesrcvd> is the number of data bytes sent to that host.
# <starttime> is the timestamp of the first appearance of that host.
# <stoptime> is the timestamp of the last appearance of that host.
#
# (be sure to use the -S flag to tcpdump to get absolute sequence numbers)

$5 !~ /[SFR]/	{
	# no SYNs, no FINs, no RSTs (why not?)

	# sender name
	n = split($2, array, ".")
	x = ""
	dot = ""
	for (i = 1; i < n; i++) {
		x = sprintf("%s%s%s", x, dot, array[i])
		dot = "."
	}
	sender = x

	# receiver name
	n = split($4, array, ".")
	x = ""
	dot = ""
	for (i = 1; i < n; i++) {
		x = sprintf("%s%s%s", x, dot, array[i])
		dot = "."
	}
	receiver = x

	# number of data bytes
	if ($6 !~ /^ack/) {
		n = split($6, array, "(")
		n = split(array[2], array, ")")
		bytes = array[1]
	} else {
		bytes = 0
	}

	# update sender statistics
	if (seen[sender] == 0) {
		seen[sender] = 1
		hostcnt++
		hosts[hostcnt] = sender
		starttime[sender] = $1
	}
	stoptime[sender] = $1
	pktssent[sender]++
	bytessent[sender] += bytes

	# update receiver statistics
	if (seen[receiver] == 0) {
		seen[receiver] = 1
		hostcnt++
		hosts[hostcnt] = receiver
		starttime[receiver] = $1
	}
	stoptime[receiver] = $1
	pktsrcvd[receiver]++
	bytesrcvd[receiver] += bytes
#	printf("%s %d %d %d %d %s %s\n",
#		sender, pktssent[sender], pktsrcvd[sender],
#		bytessent[sender], bytesrcvd[sender],
#		starttime[sender], stoptime[sender])
#	printf("%s %d %d %d %d %s %s\n",
#		receiver, pktssent[receiver], pktsrcvd[receiver],
#		bytessent[receiver], bytesrcvd[receiver],
#		starttime[receiver], stoptime[receiver])
}

END {
# do output
# <host> <pktssent> <pktsrcvd> <bytessent> <bytesrcvd> <starttime> <stoptime>
	for (i = 1; i <= hostcnt; i++) {
		host = hosts[i]
		printf("%s %d %d %d %d %s %s\n",
			host, pktssent[host], pktsrcvd[host],
			bytessent[host], bytesrcvd[host],
			starttime[host], stoptime[host])
	}
}
